/**
 * @copyright Copyright 2023 RnD Center "ELVEES", JSC
 */

#include <glib.h>
#include <gst/gst.h>
#include <gst/rtsp-server/rtsp-server.h>

#define DEFAULT_SRC                                                            \
  "videotestsrc ! video/x-raw,format=NV12,width=1280,height=720 ! "            \
  "interpipesink name=src"
#define DEFAULT_SINKS                                                          \
  "{"                                                                          \
  "\"/main\": \"interpipesrc stream-sync=restart-ts is-live=true "             \
  "listen-to=src ! queue "                                                     \
  "! x264enc speed-preset=ultrafast tune=zerolatency ! rtph264pay "            \
  "name=pay0\", "                                                              \
  "\"/secondary\": \"interpipesrc stream-sync=restart-ts is-live=true "        \
  "listen-to=src ! queue ! videoscale "                                        \
  "! video/x-raw,format=NV12,width=640,height=360 ! queue ! jpegenc ! "        \
  "rtpjpegpay name=pay0\""                                                     \
  "}"
#define DEFAULT_RTSP_PORT "8554"

static char *_src = DEFAULT_SRC;
static char *_sinks = DEFAULT_SINKS;
static char *_port = DEFAULT_RTSP_PORT;

static GOptionEntry entries[] = {
    {"src", 's', 0, G_OPTION_ARG_STRING, &_src,
     "Source pipeline (default: " DEFAULT_SRC ")", "SOURCE"},
    {"sinks", 'k', 0, G_OPTION_ARG_STRING, &_sinks,
     "Dict in JSON format {\"RTSP path\": \"RTSP bin\"} "
     "(default: " DEFAULT_SINKS ")",
     "SINKS"},
    {"port", 'p', 0, G_OPTION_ARG_STRING, &_port,
     "Port to listen on (default: " DEFAULT_RTSP_PORT ")", "PORT"},
    {NULL}};

int main(int argc, char *argv[]) {
  g_autoptr(GError) error = NULL;
  g_autoptr(GOptionContext) optctx =
      g_option_context_new("- Multistream RTSP Server");
  g_option_context_add_main_entries(optctx, entries, NULL);
  g_option_context_add_group(optctx, gst_init_get_option_group());
  if (!g_option_context_parse(optctx, &argc, &argv, &error)) {
    g_printerr("Error parsing options: %s\n", error->message);
    return -1;
  }

  GstElement *pipeline = gst_parse_launch(_src, &error);
  if (error) {
    g_printerr("Failed to parse src pipeline: %s\n", error->message);
    return -1;
  }

  if (gst_element_set_state(pipeline, GST_STATE_PLAYING) ==
      GST_STATE_CHANGE_FAILURE) {
    g_printerr("Failed to start source pipeline\n");
    return -1;
  }
  g_usleep(1e6);
  g_print("\n");

  GMainLoop *loop = g_main_loop_new(NULL, FALSE);
  GstRTSPServer *server = gst_rtsp_server_new();
  g_object_set(server, "service", _port, NULL);
  g_autoptr(GstRTSPMountPoints) mounts =
      gst_rtsp_server_get_mount_points(server);

  GVariant *sinks = g_variant_new_parsed(_sinks);
  GVariantIter iter;
  g_autofree gchar *path = NULL, *bin = NULL;
  g_variant_iter_init(&iter, sinks);
  while (g_variant_iter_loop(&iter, "{ss}", &path, &bin)) {
    GstRTSPMediaFactory *factory = gst_rtsp_media_factory_new();
    gst_rtsp_media_factory_set_launch(factory, bin);
    gst_rtsp_media_factory_set_shared(factory, TRUE);
    gst_rtsp_mount_points_add_factory(mounts, path, factory);
    g_print("Add RTSP mount point with URL rtsp://%s:%s%s\n", g_get_host_name(),
            _port, path);
  }

  gst_rtsp_server_attach(server, NULL);
  g_main_loop_run(loop);
  return 0;
}
